var fs = require('fs');
var request = require('request');
var path = require("path");

var System = function () {
    var version = "1.7";

    var AUDIO_EXTENSION = ".ogg";
    var IMAGE_EXTENSION = ".jpg";

    var isDir = function(path) {
        try {
            return fs.statSync(path).isDirectory();
        }
        catch (e) {
            return false;
        }
    }

    var isFile = function(path) {
        try {
            return fs.statSync(path).isFile();
        }
        catch (e) {
            return false;
        }
    }

    var makeDir = function (path) {
        if (!isDir(path)) {
            fs.mkdirSync(path);
        }
    }

    var download = function(uri, path, callback) { console.log("download:" + path);
        request.head(uri, function(err, res, body){
            request(uri).pipe(fs.createWriteStream(path)).on('close', callback);
        });
    }

    var getAppDir = function () {
        return path.dirname(process.execPath);
    }

    var getDbFolder = function () {
        return getDataFolder() + "/db"
    }

    var getDataFolder = function () {
        return getAppDir() + "/data"
    }

    var getPicFolder = function () {
        return getDataFolder() + "/pic"
    }

    var getAudioFolder = function () {
        return getDataFolder() + "/audio"
    }

    var getImagesFolder = function () {
        return getDataFolder() + "/image"
    }

    var downloadImage = function(url, name) {
        var filePath = getImagesFolder() + "/" + name + IMAGE_EXTENSION;
        download(url, filePath);
    }

    var downloadAudio = function(url, name) {
        var filePath = getAudioFolder() + "/" + name + AUDIO_EXTENSION;
        download(url, filePath);
    }

    var downloadPic = function(url, name) {
        var filePath = getPicFolder() + "/" + name + IMAGE_EXTENSION;
        download(url, filePath);
    }

    return {
        audioExtension: function () {
            return AUDIO_EXTENSION;
        },
        imageExtension: function () {
            return IMAGE_EXTENSION;
        },

        getVersion: function () {
            return version;
        },

        isDir: isDir,
        isFile: isFile,
        getAppDir: getAppDir,
        getDbFolder: getDbFolder,
        getPicFolder: getPicFolder,
        getImagesFolder: getImagesFolder,
        getAudioFolder: getAudioFolder,
        downloadImage: downloadImage,

        downloadAudio: function(url, name) {
            download(url, name, function() {
                console.log('done');
            });
        }
    }
}();

module.exports = System;