var Server = (function () {
    "use strict";

    var soc;
    var is_kill = false;

    var start = function() {
        if (soc == null) {
            soc = new SocClient(this);
        }

        soc.connect();
    }

    var getIp = function () {
        return DB.get(DB.SERVER_IP);
    }

    var getPort = function () {
        var port = DB.get(DB.SERVER_PORT);
        if (!port) {
            port = Server.server_port;
        }

        return port;
    }

    var getDeviceId = function () {
        return soc.getDeviceId();
    }

    var switchPort = function(port) {
        DB.set(DB.SERVER_PORT, port);
        soc.close();
    }

    var close = function () {
        is_kill = true;
        soc.close();
    }

    var canSend = function() {
        if (!soc.isConnected()) {
            ui.loadingStop();
            ui.dialogErrorSimple(_t('socket_not_connected'));
            soc.connect();
            return false;
        }

        return true;
    }

    var sending = function(data, msg) {
        if (canSend()) {
            ui.loadingStart(!msg?_t('consultando'):msg);
            soc.send(data);
        }
    }

    var send = function(data) {
        if (canSend()) {
            soc.send(data);
        }
    }

    var onConnect = function(data) {
        if (data.ismulti) {
            $('#input-login-id').show();
        }
        else {
            $('#input-login-id').hide();
        }

        if (is_kill) {
            is_kill = false;
            return;
        }

        LoginActivity.autoLogin();
    }

    var onMessage = function (msg) { //console.log("Server.onMessage()"); console.log(msg);
        if (msg.msg) {
            msg.msg = msg.msg.replace(/\n/g, "<br />");
        }

        api[msg.action](msg);
    }

    var onConnectionLost = function() { //console.log("Server.onConnectionLost()");

        /*// si ya hay un proceso de conexion, no ejecutamos
        if (connecting) { return; }
        connecting = true;

        // se muestra el modal
        box_connection_lost.show();

        percent = 0;
        reconnect();*/
    };

    var configServer = function (codigo) {
        if (codigo) {
            testConnection(codigo);
            return;
        }

        $.confirm({
            title: 'Configurar Servidor',
            content: '' +
            '<form action="" class="formName">' +
            '<div class="form-group">' +
            '<label>Escriba el código</label>' +
            '<input type="text" class="codigo form-control" maxlength="10" required />' +
            '</div>' +
            '</form>',
            buttons: {
                formSubmit: {
                    text: 'Aceptar',
                    btnClass: 'btn-blue',
                    action: function () {
                        var codigo = this.$content.find('.codigo').val();
                        if (codigo.length != 10) {
                            $.alert('Escriba un código válido');
                            return false;
                        }

                        testConnection(codigo);
                    }
                },
                cancel: function () {},
            },
            onContentReady: function () {
                var jc = this;
                this.$content.find('form').on('submit', function (e) {
                    e.preventDefault();
                    jc.$$formSubmit.trigger('click');
                });
            }
        });
    }

    var configEmpresa = function (codigo) { console.log("Server.configEmpresa()");
        if (codigo) {
            Server.sending({
                "action": packet.REGISTER_DEVICE,
                "key": codigo
            });
            return;
        }

        $.confirm({
            title: 'Registrar Terminal',
            content: '' +
            '<form action="" class="formName">' +
            '<div class="form-group">' +
            '<label>Escriba el código</label>' +
            '<input type="text" class="codigo form-control" maxlength="12" required />' +
            '</div>' +
            '</form>',
            buttons: {
                formSubmit: {
                    text: 'Aceptar',
                    btnClass: 'btn-blue',
                    action: function () {
                        var codigo = this.$content.find('.codigo').val();
                        if(codigo.length != 12) {
                            $.alert('Escriba un código válido');
                            return false;
                        }

                        Server.sending({
                            "action": packet.REGISTER_DEVICE,
                            "key": codigo
                        });
                    }
                },
                cancel: function () {},
            },
            onContentReady: function () {
                var jc = this;
                this.$content.find('form').on('submit', function (e) {
                    e.preventDefault();
                    jc.$$formSubmit.trigger('click');
                });
            }
        });
    }

    var testConnection = async function(codigo) {
        ui.loadingStart(_t('connecting'));
        try {
            var ip = await soc.testConnect(codigo);
            if (!ip) {
                ui.dialogErrorSimple('Código incorrecto!');
                return;
            }

            soc.connect();
            ui.dialogSuccess('Servidor configurado');
            DB.set(DB.SERVER_IP, ip);
        }
        catch(e) {
            ui.dialogErrorSimple('Código incorrecto!');
            console.log(e);
        }
        ui.loadingStop();
    }

    return {
        close: close,
        start: start,
        send: send,
        sending: sending,
        onConnect: onConnect,
        onConnectionLost: onConnectionLost,
        onMessage: onMessage,
        getDeviceId: getDeviceId,
        getPort: getPort,
        configServer: configServer,
        configEmpresa: configEmpresa,
        switchPort: switchPort,
        appVersion: function () {
            return soc.getVersion();
        },
        getIp: function () {
            return '70.35.207.61';
        },
        server_port: 7000,
        url_update_check: "/app/checkupdate",
        url_update: "/app/getapk"
    }
}());