"use strict";

var http = require('http');
var net = require('net');
var os = require('os');
var zlib = require('zlib');
var machine = require('node-machine-id');

var SocClient = function(master) {
    "use strict";

    var self = this;

    var RECONNECT_INTERVAL = 5000;

    var APP_VERSION_NAME = '1.7';
    var APP_VERSION_CODE = 17;

    var STATUS_CONNECTING = 'connecting';
    var STATUS_CONNECTED = 'connected';
    var STATUS_DISCONNECTED = 'disconnected';
    var STATUS_RECONNECTING = 'reconnecting';

    var client = null;
    var status = null;
    var timer = null;
    var connecting_count = 0;
    var reconnecting_count = 0;

    var dataCompleted = '';
    var i,json,json2,ismulti;
    this.connect = function(fromTimmer) { console.log("SocClient.connect() port:"+master.getPort());

        if (!fromTimmer && (status === STATUS_CONNECTED || status === STATUS_CONNECTING || status === STATUS_RECONNECTING)) {
            return;
        }

        clearTimeout(timer);
        destroy();
        status = STATUS_CONNECTING;
        client = net.connect(master.getPort(), master.getIp(), function() { console.log('connected!');
            status = STATUS_CONNECTED;
            connecting_count = 0;
            reconnecting_count = 0;

            self.send({
                "action": "CONNECT",
                "device": self.getDeviceId(),
                "device_version": 'PC',
                "marca": os.platform(),
                "modelo": os.arch(),
                "mac": machine.machineIdSync({original: true}),
                "version": APP_VERSION_NAME
            });
        });

        client.on('data', function(data) {
            //data = decompress(data);
            data = data.toString().trim(); console.log("soc_recv: " + data);

            if (data.slice(-1) === '|') {
                dataCompleted += data.slice(0,-1);
            }
            else {
                dataCompleted += data;
                return;
            }

            data = dataCompleted;
            dataCompleted = '';
            ismulti = false;

            if (data.indexOf('|') >= 0) {
                ismulti = true;
                json2 = [];
                var datas = data.split('|');
                for (i=0; i<datas.length; i++) {
                    json2.push(JSON.parse(datas[i]));
                }
                //logger.w(data, 'PACKET_MULTIPLE', serial);
            }
            else {
                json = JSON.parse(data);
            }

            if (ismulti) {
                for (i=0; i<json2.length; i++) {
                    json = json2[i];
                    master.onMessage(json);
                }
            }
            else {
                master.onMessage(json);
            }
        });

        client.on('error', function(err){
            console.log("Error: "+err.message);
        });

        client.on('end', function() {
            console.log('client.end()');
        });

        client.on('close', function() { console.log('soc_on_close status:' + status);

            if (status == STATUS_CONNECTED) {
                console.log('disconnected from server (close)');
                status = STATUS_CONNECTING;
                master.onConnectionLost();
            }
            else if (status == STATUS_RECONNECTING) {
                connecting_count++;
                console.log("intento1 de conexion fallido # "+connecting_count);
            }
            else if (status == STATUS_CONNECTING) {
                reconnecting_count++;
                console.log("intento2 de reconexion fallido # "+reconnecting_count);
            }

            timer = setTimeout(function() {self.connect(true);}, RECONNECT_INTERVAL);
        });
    }

    this.isConnected = function() {
        return status === STATUS_CONNECTED;
    }

    this.send = function(data) { console.log("SocClient.send(): " + JSON.stringify(data));
        //client.write(compress(JSON.stringify(data) + "|"));
        client.write(JSON.stringify(data) + "|");
    }

    this.close = function() {
        destroy();
    }

    this.getDeviceId = function() {
        return machine.machineIdSync().substr(0,20);
    }

    var compress = function(data) {
        return zlib.deflateSync(data);
    }

    var decompress = function(data) {
        return zlib.inflateSync(data);
    }

    var destroy = function() { //console.log("SocClient.destroy()");
        if (client == null) { return; }
        client.destroy();
        client = null;
    }

    var do_connect = function() {
        connect();
    }

    var get_status = function() {
        return status;
    }




    this.testConnect = async function(ip) {
        ip = int2ip(ip); console.log("SocClient.testConnec() ip:"+ip);

        return new Promise(function(resolve, reject) {
            var res = net.connect(master.server_port, ip, function() {
                resolve(ip);
            });

            res.on('error', function(err) {
                reject('error');
            });
        });

        console.log("bbb");
    }

    this.getVersion = function() {
        return APP_VERSION_NAME;
    }

    var int2ip = function(ipInt) {
        return ( (ipInt>>>24) +'.' + (ipInt>>16 & 255) +'.' + (ipInt>>8 & 255) +'.' + (ipInt & 255) );
    }
}

module.exports = SocClient;